@@ -34,37 +34,72 @@ def parse_arguments(progname='requake'):
     parser.add_argument(
         '-v', '--version', action='version',
         version='%(prog)s {}'.format(get_versions()['version']))
+    # --- sample_config
     subparser.add_parser(
         'sample_config',
         help='write sample config file to current directory and exit'
     )
+    # ---
+    # --- scan_catalog
     subparser.add_parser(
         'scan_catalog',
         help='scan an existing catalog for earthquake pairs'
     )
+    # ---
     # subparser.add_parser(
     #     'scan_template',
     #     help='scan a continuous waveform stream using a template'
     # )
+    # --- plot_pair
     plotpair = subparser.add_parser(
         'plot_pair',
         help='plot traces for a given event pair'
     )
     plotpair.add_argument('evid1')
     plotpair.add_argument('evid2')
+    # ---
+    # --- build_families
     subparser.add_parser(
         'build_families',
         help='build families of repeating earthquakes from a catalog of pairs'
     )
-    plotfamilies = subparser.add_parser(
-        'plot_families',
-        help='plot traces for one ore more event families'
+    # ---
+    # --- longerthan
+    #     a parent parser for the "longerthan" option,
+    #     used by several subparsers
+    longerthan = argparse.ArgumentParser(add_help=False)
+    longerthan.add_argument(
+        '-l', '--longerthan', type=str, default=0, metavar='DURATION',
+        help='only use families lasting longer than this value. '
+             'You can specify DURATION in days (e.g., 100d) '
+             'or in years (e.g., 2.5y).'
     )
-    plotfamilies.add_argument(
+    # ---
+    # --- familynumbers
+    #     a parent parser for the "family_numbers" option,
+    #     used by several subparsers
+    familynumbers = argparse.ArgumentParser(add_help=False)
+    familynumbers.add_argument(
         'family_numbers',
-        help='family_numbers to plot. It can be a single number, '
+        help='family numbers to use. It can be a single number, '
              'a comma-separated list or a hyphen-separated number range. '
              'Use "all" to specify all the families.')
+    # ---
+    # --- traceid
+    #     a parent parser for the "traceid" option,
+    #     used by several subparsers
+    traceid = argparse.ArgumentParser(add_help=False)
+    traceid.add_argument(
+        '-t', '--traceid', type=str, default=None,
+        help='use this traceid instead of the default one for the family.'
+    )
+    # ---
+    # --- plot_families
+    plotfamilies = subparser.add_parser(
+        'plot_families',
+        parents=[longerthan, familynumbers, traceid],
+        help='plot traces for one ore more event families'
+    )
     plotfamilies.add_argument(
         '-s', '--starttime', type=float, default=None,
         help='start time, in seconds relative to trace start, for the plot.'
@@ -73,22 +108,11 @@ def parse_arguments(progname='requake'):
         '-e', '--endtime', type=float, default=None,
         help='end time, in seconds relative to trace start, for the plot.'
     )
-    plotfamilies.add_argument(
-        '-a', '--arrivals', action='store_true',
-        help='plot theoretical P and S arrivals.'
-    )
-    plotfamilies.add_argument(
-        '-t', '--traceid', type=str, default=None,
-        help='plot using this traceid.'
-    )
-    plotfamilies.add_argument(
-        '-l', '--longerthan', type=str, default=0, metavar='DURATION',
-        help='only use families lasting longer than this value. '
-             'You can specify DURATION in days (e.g., 100d) '
-             'or in years (e.g., 2.5y).'
-    )
+    # ---
+    # --- plot_timespans
     timespans = subparser.add_parser(
         'plot_timespans',
+        parents=[longerthan],
         help='plot family timespans'
     )
     timespans.add_argument(
@@ -98,22 +122,15 @@ def parse_arguments(progname='requake'):
              'distance_from. If not specified, the config value '
              '"sort_families_by" will be used.'
     )
-    timespans.add_argument(
-        '-l', '--longerthan', type=str, default=0, metavar='DURATION',
-        help='only use families lasting longer than this value. '
-             'You can specify DURATION in days (e.g., 100d) '
-             'or in years (e.g., 2.5y).'
-    )
-    mapfamilies = subparser.add_parser(
+    # ---
+    # --- map_families
+    subparser.add_parser(
         'map_families',
+        parents=[longerthan],
         help='plot families on a map'
     )
-    mapfamilies.add_argument(
-        '-l', '--longerthan', type=str, default=0, metavar='DURATION',
-        help='only use families lasting longer than this value. '
-             'You can specify DURATION in days (e.g., 100d) '
-             'or in years (e.g., 2.5y).'
-    )
+    # ---
+    # --- flag_family
     flagfamily = subparser.add_parser(
         'flag_family',
         help='flag a family of repeating earthquakes as valid or not valid. '
@@ -125,6 +142,14 @@ def parse_arguments(progname='requake'):
         help='"true" (or "t") to flag family as valid, '
              '"false" (or "f") to flag family as not valid.'
     )
+    # ---
+    # --- build_templates
+    subparser.add_parser(
+        'build_templates',
+        parents=[longerthan, familynumbers, traceid],
+        help='build waveform templates for one or more event families'
+    )
+    # ---
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
     if args.action is None:
