@@ -12,7 +12,8 @@ Functions for downloading and processing waveforms.
 import logging
 logger = logging.getLogger(__name__.split('.')[-1])
 import numpy as np
-from obspy import Inventory, Stream
+from itertools import combinations
+from obspy import Inventory, Stream, UTCDateTime
 from obspy.geodetics import gps2dist_azimuth, locations2degrees
 from obspy.taup import TauPyModel
 model = TauPyModel(model='ak135')
@@ -20,7 +21,7 @@ from obspy.signal.cross_correlation import correlate, xcorr_max
 from .rq_setup import rq_exit
 
 
-def get_metadata(config):
+def _get_metadata(config):
     """Download metadata for the trace_ids specified in config file."""
     logger.info('Downloading station metadata...')
     inv = Inventory()
@@ -58,6 +59,8 @@ def get_metadata(config):
 
 def _get_trace_id(config, ev):
     """Get trace id to use with ev."""
+    if config.inventory is None:
+        _get_metadata(config)
     trace_ids = config.catalog_trace_id
     if len(trace_ids) == 1:
         return trace_ids[0]
@@ -83,8 +86,10 @@ def _get_trace_id(config, ev):
     return closest_trace
 
 
-def download_and_process_waveform(config, ev):
-    """Download and process waveform for a given event at a given trace_id."""
+def get_waveform(config, ev):
+    """Download waveform for a given event at a given trace_id."""
+    if config.inventory is None:
+        _get_metadata(config)
     evid = ev.evid
     ev_lat = ev.lat
     ev_lon = ev.lon
@@ -130,14 +135,9 @@ def download_and_process_waveform(config, ev):
     )
     # webservices sometimes return longer traces: trim to be sure
     st.trim(starttime=t0, endtime=t1)
-    st.detrend(type='demean')
-    st.taper(max_percentage=0.05)
-    st.filter(
-        type='bandpass',
-        freqmin=config.cc_freq_min,
-        freqmax=config.cc_freq_max)
     st.merge(fill_value='interpolate')
     tr = st[0]
+    tr.detrend(type='demean')
     tr.stats.evid = evid
     tr.stats.ev_lat = ev_lat
     tr.stats.ev_lon = ev_lon
@@ -145,6 +145,7 @@ def download_and_process_waveform(config, ev):
     tr.stats.orig_time = orig_time
     tr.stats.mag = mag
     tr.stats.mag_type = mag_type
+    tr.stats.coords = coords
     tr.stats.dist_deg = dist_deg
     tr.stats.distance = distance
     tr.stats.P_arrival_time = P_arrival_time
@@ -152,6 +153,18 @@ def download_and_process_waveform(config, ev):
     return tr
 
 
+def process_waveforms(config, st):
+    """Demean and filter a waveform trace or stream."""
+    st = st.copy()
+    st.detrend(type='demean')
+    st.taper(max_percentage=0.05)
+    st.filter(
+        type='bandpass',
+        freqmin=config.cc_freq_min,
+        freqmax=config.cc_freq_max)
+    return st
+
+
 skipped_evids = list()
 tr_cache = dict()
 old_cache_key = None
@@ -160,7 +173,7 @@ old_cache_key = None
 def get_waveform_pair(config, pair):
     """Download traces for a given pair."""
     if config.inventory is None:
-        get_metadata(config)
+        _get_metadata(config)
     ev1, ev2 = pair
     ev1.trace_id = ev2.trace_id = _get_trace_id(config, ev1)
     st = Stream()
@@ -186,7 +199,7 @@ def get_waveform_pair(config, pair):
         except KeyError:
             pass
         try:
-            tr = download_and_process_waveform(config, ev)
+            tr = get_waveform(config, ev)
             tr_cache[cache_key] = tr
             st.append(tr)
         except Exception as m:
@@ -214,9 +227,12 @@ def cc_waveform_pair(config, tr1, tr2):
             '{} {} - The two traces have a different sampling interval.'
             'Skipping pair.'.format(evid1, evid2))
         raise
+    tr1 = process_waveforms(config, tr1)
+    tr2 = process_waveforms(config, tr2)
     shift = int(config.cc_max_shift/dt1)
     cc = correlate(tr1, tr2, shift)
-    lag, cc_max = xcorr_max(cc)
+    abs_max = bool(config.cc_allow_negative)
+    lag, cc_max = xcorr_max(cc, abs_max)
     lag_sec = lag*dt1
     return lag, lag_sec, cc_max
 
@@ -236,11 +252,66 @@ def align_pair(config, tr1, tr2):
     else:
         data2 = tr2.data
     tr2.data = data2
+    tr2.stats.P_arrival_time += lag_sec
+    tr2.stats.S_arrival_time += lag_sec
     return lag, lag_sec, cc_max
 
 
 def align_traces(config, st):
     """Align traces in stream using cross-correlation."""
-    tr1 = st[0]
-    for tr2 in st[1:]:
-        align_pair(config, tr1, tr2)
+    for tr in st:
+        tr.stats.cc_mean = 0
+        tr.stats.cc_npairs = 0
+    for tr1, tr2 in combinations(st, 2):
+        _, _, cc_max = align_pair(config, tr1, tr2)
+        tr1.stats.cc_mean += cc_max
+        tr1.stats.cc_npairs += 1
+        tr2.stats.cc_mean += cc_max
+        tr2.stats.cc_npairs += 1
+    for tr in st:
+        tr.stats.cc_mean /= tr.stats.cc_npairs
+
+
+def build_template(config, st, family):
+    """
+    Build template by averaging traces.
+
+    Assumes that the stream is realigned.
+    """
+    tr_template = st[0].copy()
+    if config.trace_average_from_normalized_traces:
+        tr_template.data /= abs(tr_template.max())
+    P_arrival = tr_template.stats.P_arrival_time - tr_template.stats.starttime
+    S_arrival = tr_template.stats.S_arrival_time - tr_template.stats.starttime
+    for tr in st[1:]:
+        data = tr.data
+        if config.trace_average_from_normalized_traces:
+            data /= abs(tr.max())
+        tr_template.data += data
+        P_arrival += tr.stats.P_arrival_time - tr.stats.starttime
+        S_arrival += tr.stats.S_arrival_time - tr.stats.starttime
+    tr_template.data /= len(st)
+    P_arrival /= len(st)
+    S_arrival /= len(st)
+    tr_template.stats.evid = 'average{:02d}'.format(family.number)
+    tr_template.stats.ev_lat = family.lat
+    tr_template.stats.ev_lon = family.lon
+    tr_template.stats.ev_depth = family.depth
+    tr_template.stats.orig_time = UTCDateTime('1900/01/01T00:00:00')
+    tr_template.stats.mag = 0
+    tr_template.stats.mag_type = ''
+    trace_lat = tr_template.stats.coords['latitude']
+    trace_lon = tr_template.stats.coords['longitude']
+    dist_deg = locations2degrees(
+        trace_lat, trace_lon, family.lat, family.lon)
+    distance, _, _ = gps2dist_azimuth(
+        trace_lat, trace_lon, family.lat, family.lon)
+    distance /= 1e3
+    tr_template.stats.dist_deg = dist_deg
+    tr_template.stats.distance = distance
+    tr_template.stats.starttime = UTCDateTime('1900/01/01T00:00:00')
+    tr_template.stats.P_arrival_time = tr_template.stats.starttime + P_arrival
+    tr_template.stats.S_arrival_time = tr_template.stats.starttime + S_arrival
+    tr_template.stats.cc_mean = 0
+    tr_template.stats.cc_npairs = 0
+    st.append(tr_template)
